@@ -1,4 +1,4 @@
-# Copyright 2009-2012 10gen, Inc.
+# Copyright 2009-2014 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,8 +23,10 @@ The :mod:`gridfs` package is an implementation of GridFS on top of
 from gridfs.errors import (NoFile,
                            UnsupportedAPI)
 from gridfs.grid_file import (GridIn,
-                              GridOut)
-from pymongo import (ASCENDING,
+                              GridOut,
+                              GridOutCursor)
+from pymongo import (MongoClient,
+                     ASCENDING,
                      DESCENDING)
 from pymongo.database import Database
 
@@ -32,7 +34,7 @@ from pymongo.database import Database
 class GridFS(object):
     """An instance of GridFS on top of a single Database.
     """
-    def __init__(self, database, collection="fs"):
+    def __init__(self, database, collection="fs", _connect=True):
         """Create a new instance of :class:`GridFS`.
 
         Raises :class:`TypeError` if `database` is not an instance of
@@ -54,12 +56,27 @@ class GridFS(object):
         self.__collection = database[collection]
         self.__files = self.__collection.files
         self.__chunks = self.__collection.chunks
-        connection = database.connection
-        if not hasattr(connection, 'is_primary') or connection.is_primary:
+        if _connect:
+            self.__ensure_index_files_id()
+
+    def __is_secondary(self):
+        client = self.__database.connection
+
+        # Connect the client, so we know if it's connected to the primary.
+        client._ensure_connected()
+        return isinstance(client, MongoClient) and not client.is_primary
+
+    def __ensure_index_files_id(self):
+        if not self.__is_secondary():
             self.__chunks.ensure_index([("files_id", ASCENDING),
                                         ("n", ASCENDING)],
                                        unique=True)
 
+    def __ensure_index_filename(self):
+        if not self.__is_secondary():
+            self.__files.ensure_index([("filename", ASCENDING),
+                                       ("uploadDate", DESCENDING)])
+
     def new_file(self, **kwargs):
         """Create a new file in GridFS.
 
@@ -76,6 +93,8 @@ class GridFS(object):
 
         .. versionadded:: 1.6
         """
+        # No need for __ensure_index_files_id() here; GridIn ensures
+        # the (files_id, n) index when needed.
         return GridIn(self.__collection, **kwargs)
 
     def put(self, data, **kwargs):
@@ -86,7 +105,7 @@ class GridFS(object):
           try:
               f = new_file(**kwargs)
               f.write(data)
-          finally
+          finally:
               f.close()
 
         `data` can be either an instance of :class:`str` (:class:`bytes`
@@ -176,11 +195,7 @@ class GridFS(object):
            Accept keyword arguments to find files by custom metadata.
         .. versionadded:: 1.9
         """
-        connection = self.__database.connection
-        if not hasattr(connection, 'is_primary') or connection.is_primary:
-            self.__files.ensure_index([("filename", ASCENDING),
-                                       ("uploadDate", DESCENDING)])
-
+        self.__ensure_index_filename()
         query = kwargs
         if filename is not None:
             query["filename"] = filename
@@ -237,6 +252,7 @@ class GridFS(object):
 
         .. versionadded:: 1.6
         """
+        self.__ensure_index_files_id()
         self.__files.remove({"_id": file_id},
                             **self.__files._get_wc_override())
         self.__chunks.remove({"files_id": file_id})
@@ -245,15 +261,121 @@ class GridFS(object):
         """List the names of all files stored in this instance of
         :class:`GridFS`.
 
+        An index on ``{filename: 1, uploadDate: -1}`` will
+        automatically be created when this method is called the first
+        time.
+
+        .. versionchanged:: 2.7
+           ``list`` ensures an index, the same as ``get_version``.
+
         .. versionchanged:: 1.6
            Removed the `collection` argument.
         """
-        return self.__files.distinct("filename")
+        self.__ensure_index_filename()
+
+        # With an index, distinct includes documents with no filename
+        # as None.
+        return [
+            name for name in self.__files.distinct("filename")
+            if name is not None]
+
+    def find_one(self, spec_or_id=None, *args, **kwargs):
+        """Get a single file from gridfs.
+
+        All arguments to :meth:`find` are also valid arguments for
+        :meth:`find_one`, although any `limit` argument will be
+        ignored. Returns a single :class:`~gridfs.grid_file.GridOut`,
+        or ``None`` if no matching file is found. For example::
+
+            file = fs.find_one({"filename": "lisa.txt"})
+
+        :Parameters:
+          - `spec_or_id` (optional): a dictionary specifying
+            the query to be performing OR any other type to be used  as
+            the value for a query for ``"_id"`` in the file collection.
+          - `*args` (optional): any additional positional arguments are
+            the same as the arguments to :meth:`find`.
+          - `**kwargs` (optional): any additional keyword arguments
+            are the same as the arguments to :meth:`find`.
+        """
+        if spec_or_id is not None and not isinstance(spec_or_id, dict):
+            spec_or_id = {"_id": spec_or_id}
+
+        for f in self.find(spec_or_id, *args, **kwargs):
+            return f
+
+        return None
+
+    def find(self, *args, **kwargs):
+        """Query GridFS for files.
+
+        Returns a cursor that iterates across files matching
+        arbitrary queries on the files collection. Can be combined
+        with other modifiers for additional control. For example::
+
+          for grid_out in fs.find({"filename": "lisa.txt"}, timeout=False):
+              data = grid_out.read()
+
+        would iterate through all versions of "lisa.txt" stored in GridFS.
+        Note that setting timeout to False may be important to prevent the
+        cursor from timing out during long multi-file processing work.
+
+        As another example, the call::
+
+          most_recent_three = fs.find().sort("uploadDate", -1).limit(3)
+
+        would return a cursor to the three most recently uploaded files
+        in GridFS.
+
+        Follows a similar interface to
+        :meth:`~pymongo.collection.Collection.find`
+        in :class:`~pymongo.collection.Collection`.
+
+        :Parameters:
+          - `spec` (optional): a SON object specifying elements which
+            must be present for a document to be included in the
+            result set
+          - `skip` (optional): the number of files to omit (from
+            the start of the result set) when returning the results
+          - `limit` (optional): the maximum number of results to
+            return
+          - `timeout` (optional): if True (the default), any returned
+            cursor is closed by the server after 10 minutes of
+            inactivity. If set to False, the returned cursor will never
+            time out on the server. Care should be taken to ensure that
+            cursors with timeout turned off are properly closed.
+          - `sort` (optional): a list of (key, direction) pairs
+            specifying the sort order for this query. See
+            :meth:`~pymongo.cursor.Cursor.sort` for details.
+          - `max_scan` (optional): limit the number of file documents
+            examined when performing the query
+          - `read_preference` (optional): The read preference for
+            this query.
+          - `tag_sets` (optional): The tag sets for this query.
+          - `secondary_acceptable_latency_ms` (optional): Any replica-set
+            member whose ping time is within secondary_acceptable_latency_ms of
+            the nearest member may accept reads. Default 15 milliseconds.
+            **Ignored by mongos** and must be configured on the command line.
+            See the localThreshold_ option for more information.
+          - `compile_re` (optional): if ``False``, don't attempt to compile
+            BSON regex objects into Python regexes. Return instances of
+            :class:`~bson.regex.Regex` instead.
+
+        Raises :class:`TypeError` if any of the arguments are of
+        improper type. Returns an instance of
+        :class:`~gridfs.grid_file.GridOutCursor`
+        corresponding to this query.
+
+        .. versionadded:: 2.7
+        .. mongodoc:: find
+        .. _localThreshold: http://docs.mongodb.org/manual/reference/mongos/#cmdoption-mongos--localThreshold
+        """
+        return GridOutCursor(self.__collection, *args, **kwargs)
 
     def exists(self, document_or_id=None, **kwargs):
         """Check if a file exists in this instance of :class:`GridFS`.
 
-        The file to check for can be specified by the value of it's
+        The file to check for can be specified by the value of its
         ``_id`` key, or by passing in a query document. A query
         document can be passed in as dictionary, or by using keyword
         arguments. Thus, the following three calls are equivalent:
