@@ -1,4 +1,4 @@
-# Copyright 2009-2012 10gen, Inc.
+# Copyright 2009-2014 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,8 +21,9 @@ from bson.code import Code
 from bson.son import SON
 from pymongo import helpers, message, read_preferences
 from pymongo.read_preferences import ReadPreference, secondary_ok_commands
-from pymongo.errors import (InvalidOperation,
-                            AutoReconnect)
+from pymongo.errors import (AutoReconnect,
+                            InvalidOperation,
+                            OperationFailure)
 
 _QUERY_OPTIONS = {
     "tailable_cursor": 2,
@@ -55,6 +56,15 @@ class _SocketManager:
             self.pool.maybe_return_socket(self.sock)
             self.sock, self.pool = None, None
 
+    def error(self):
+        """Clean up after an error on the managed socket.
+        """
+        if self.sock:
+            self.sock.close()
+
+        # Return the closed socket to avoid a semaphore leak in the pool.
+        self.close()
+
 
 # TODO might be cool to be able to do find().include("foo") or
 # find().exclude(["bar", "baz"]) or find().slice("a", 1, 2) as an
@@ -67,11 +77,10 @@ class Cursor(object):
                  timeout=True, snapshot=False, tailable=False, sort=None,
                  max_scan=None, as_class=None, slave_okay=False,
                  await_data=False, partial=False, manipulate=True,
-                 read_preference=ReadPreference.PRIMARY, tag_sets=[{}],
-                 secondary_acceptable_latency_ms=None, exhaust=False,
-                 _must_use_master=False, _uuid_subtype=None,
-                 _first_batch=None, _cursor_id=None,
-                 **kwargs):
+                 read_preference=ReadPreference.PRIMARY,
+                 tag_sets=[{}], secondary_acceptable_latency_ms=None,
+                 exhaust=False, compile_re=True, _must_use_master=False,
+                 _uuid_subtype=None, **kwargs):
         """Create a new cursor.
 
         Should not be called directly by application developers - see
@@ -79,8 +88,7 @@ class Cursor(object):
 
         .. mongodoc:: cursors
         """
-        self.__id = _cursor_id
-        self.__is_command_cursor = _cursor_id is not None
+        self.__id = None
 
         if spec is None:
             spec = {}
@@ -120,7 +128,10 @@ class Cursor(object):
         self.__fields = fields
         self.__skip = skip
         self.__limit = limit
+        self.__max_time_ms = None
         self.__batch_size = 0
+        self.__max = None
+        self.__min = None
 
         # Exhaust cursor support
         if self.__collection.database.connection.is_mongos and exhaust:
@@ -144,6 +155,7 @@ class Cursor(object):
         self.__max_scan = max_scan
         self.__explain = False
         self.__hint = None
+        self.__comment = None
         self.__as_class = as_class
         self.__slave_okay = slave_okay
         self.__manipulate = manipulate
@@ -151,10 +163,11 @@ class Cursor(object):
         self.__tag_sets = tag_sets
         self.__secondary_acceptable_latency_ms = secondary_acceptable_latency_ms
         self.__tz_aware = collection.database.connection.tz_aware
+        self.__compile_re = compile_re
         self.__must_use_master = _must_use_master
         self.__uuid_subtype = _uuid_subtype or collection.uuid_subtype
 
-        self.__data = deque(_first_batch or [])
+        self.__data = deque()
         self.__connection_id = None
         self.__retrieved = 0
         self.__killed = False
@@ -184,6 +197,22 @@ class Cursor(object):
         """
         return self.__collection
 
+    @property
+    def conn_id(self):
+        """The server/client/pool this cursor lives on.
+
+        Could be (host, port), -1, or None depending on what
+        client class executed the initial query or this cursor
+        being advanced at all.
+        """
+        return self.__connection_id
+
+    @property
+    def retrieved(self):
+        """The number of documents retrieved so far.
+        """
+        return self.__retrieved
+
     def __del__(self):
         if self.__id and not self.__killed:
             self.__die()
@@ -197,7 +226,6 @@ class Cursor(object):
         be sent to the server, even if the resultant data has already been
         retrieved by this cursor.
         """
-        self.__check_not_command_cursor('rewind')
         self.__data = deque()
         self.__id = None
         self.__connection_id = None
@@ -214,25 +242,30 @@ class Cursor(object):
         unevaluated, even if the current instance has been partially or
         completely evaluated.
         """
-        return self.__clone(True)
+        return self._clone(True)
 
-    def __clone(self, deepcopy=True):
-        self.__check_not_command_cursor('clone')
-        clone = Cursor(self.__collection)
-        values_to_clone = ("spec", "fields", "skip", "limit",
+    def _clone(self, deepcopy=True):
+        clone = self._clone_base()
+        values_to_clone = ("spec", "fields", "skip", "limit", "max_time_ms",
+                           "comment", "max", "min",
                            "snapshot", "ordering", "explain", "hint",
                            "batch_size", "max_scan", "as_class", "slave_okay",
                            "manipulate", "read_preference", "tag_sets",
                            "secondary_acceptable_latency_ms",
-                           "must_use_master", "uuid_subtype", "query_flags",
-                           "kwargs")
+                           "must_use_master", "uuid_subtype", "compile_re",
+                           "query_flags", "kwargs")
         data = dict((k, v) for k, v in self.__dict__.iteritems()
                     if k.startswith('_Cursor__') and k[9:] in values_to_clone)
         if deepcopy:
-            data = self.__deepcopy(data)
+            data = self._deepcopy(data)
         clone.__dict__.update(data)
         return clone
 
+    def _clone_base(self):
+        """Creates an empty Cursor object for information to be copied into.
+        """
+        return Cursor(self.__collection)
+
     def __die(self):
         """Closes this cursor.
         """
@@ -269,10 +302,18 @@ class Cursor(object):
             operators["$explain"] = True
         if self.__hint:
             operators["$hint"] = self.__hint
+        if self.__comment:
+            operators["$comment"] = self.__comment
         if self.__snapshot:
             operators["$snapshot"] = True
         if self.__max_scan:
             operators["$maxScan"] = self.__max_scan
+        if self.__max_time_ms is not None:
+            operators["$maxTimeMS"] = self.__max_time_ms
+        if self.__max:
+            operators["$max"] = self.__max
+        if self.__min:
+            operators["$min"] = self.__min
         # Only set $readPreference if it's something other than
         # PRIMARY to avoid problems with mongos versions that
         # don't support read preferences.
@@ -353,15 +394,8 @@ class Cursor(object):
         if self.__retrieved or self.__id is not None:
             raise InvalidOperation("cannot set options after executing query")
 
-    def __check_not_command_cursor(self, method_name):
-        """Check if calling a method on this cursor is valid.
-        """
-        if self.__is_command_cursor:
-            raise InvalidOperation(
-                "cannot call %s on a command cursor" % method_name)
-
     def add_option(self, mask):
-        """Set arbitary query flags using a bitmask.
+        """Set arbitrary query flags using a bitmask.
 
         To set the tailable flag:
         cursor.add_option(2)
@@ -404,18 +438,18 @@ class Cursor(object):
     def limit(self, limit):
         """Limits the number of results to be returned by this cursor.
 
-        Raises TypeError if limit is not an instance of int. Raises
-        InvalidOperation if this cursor has already been used. The
-        last `limit` applied to this cursor takes precedence. A limit
-        of ``0`` is equivalent to no limit.
+        Raises :exc:`TypeError` if `limit` is not an integer. Raises
+        :exc:`~pymongo.errors.InvalidOperation` if this :class:`Cursor`
+        has already been used. The last `limit` applied to this cursor
+        takes precedence. A limit of ``0`` is equivalent to no limit.
 
         :Parameters:
           - `limit`: the number of results to return
 
         .. mongodoc:: limit
         """
-        if not isinstance(limit, int):
-            raise TypeError("limit must be an int")
+        if not isinstance(limit, (int, long)):
+            raise TypeError("limit must be an integer")
         if self.__exhaust:
             raise InvalidOperation("Can't use limit and exhaust together.")
         self.__check_okay_to_chain()
@@ -434,10 +468,9 @@ class Cursor(object):
            if you set batch size to 1,000,000,000, MongoDB will currently only
            return 4-16MB of results per batch).
 
-        Raises :class:`TypeError` if `batch_size` is not an instance
-        of :class:`int`. Raises :class:`ValueError` if `batch_size` is
-        less than ``0``. Raises
-        :class:`~pymongo.errors.InvalidOperation` if this
+        Raises :exc:`TypeError` if `batch_size` is not an integer.
+        Raises :exc:`ValueError` if `batch_size` is less than ``0``.
+        Raises :exc:`~pymongo.errors.InvalidOperation` if this
         :class:`Cursor` has already been used. The last `batch_size`
         applied to this cursor takes precedence.
 
@@ -446,8 +479,8 @@ class Cursor(object):
 
         .. versionadded:: 1.9
         """
-        if not isinstance(batch_size, int):
-            raise TypeError("batch_size must be an int")
+        if not isinstance(batch_size, (int, long)):
+            raise TypeError("batch_size must be an integer")
         if batch_size < 0:
             raise ValueError("batch_size must be >= 0")
         self.__check_okay_to_chain()
@@ -458,20 +491,44 @@ class Cursor(object):
     def skip(self, skip):
         """Skips the first `skip` results of this cursor.
 
-        Raises TypeError if skip is not an instance of int. Raises
-        InvalidOperation if this cursor has already been used. The last `skip`
-        applied to this cursor takes precedence.
+        Raises :exc:`TypeError` if `skip` is not an integer. Raises
+        :exc:`ValueError` if `skip` is less than ``0``. Raises
+        :exc:`~pymongo.errors.InvalidOperation` if this :class:`Cursor` has
+        already been used. The last `skip` applied to this cursor takes
+        precedence.
 
         :Parameters:
           - `skip`: the number of results to skip
         """
         if not isinstance(skip, (int, long)):
-            raise TypeError("skip must be an int")
+            raise TypeError("skip must be an integer")
+        if skip < 0:
+            raise ValueError("skip must be >= 0")
         self.__check_okay_to_chain()
 
         self.__skip = skip
         return self
 
+    def max_time_ms(self, max_time_ms):
+        """Specifies a time limit for a query operation. If the specified
+        time is exceeded, the operation will be aborted and
+        :exc:`~pymongo.errors.ExecutionTimeout` is raised. If `max_time_ms`
+        is ``None`` no limit is applied.
+
+        Raises :exc:`TypeError` if `max_time_ms` is not an integer or ``None``.
+        Raises :exc:`~pymongo.errors.InvalidOperation` if this :class:`Cursor`
+        has already been used.
+
+        :Parameters:
+          - `max_time_ms`: the time limit after which the operation is aborted
+        """
+        if not isinstance(max_time_ms, (int, long)) and max_time_ms is not None:
+            raise TypeError("max_time_ms must be an integer or None")
+        self.__check_okay_to_chain()
+
+        self.__max_time_ms = max_time_ms
+        return self
+
     def __getitem__(self, index):
         """Get a single document or a slice of documents from this cursor.
 
@@ -559,15 +616,68 @@ class Cursor(object):
         self.__max_scan = max_scan
         return self
 
+    def max(self, spec):
+        """Adds `max` operator that specifies upper bound for specific index.
+
+        :Parameters:
+          - `spec`: a list of field, limit pairs specifying the exclusive
+            upper bound for all keys of a specific index in order.
+
+        .. versionadded:: 2.7
+        """
+        if not isinstance(spec, (list, tuple)):
+            raise TypeError("spec must be an instance of list or tuple")
+
+        self.__check_okay_to_chain()
+        self.__max = SON(spec)
+        return self
+
+    def min(self, spec):
+        """Adds `min` operator that specifies lower bound for specific index.
+
+        :Parameters:
+          - `spec`: a list of field, limit pairs specifying the inclusive
+            lower bound for all keys of a specific index in order.
+
+        .. versionadded:: 2.7
+        """
+        if not isinstance(spec, (list, tuple)):
+            raise TypeError("spec must be an instance of list or tuple")
+
+        self.__check_okay_to_chain()
+        self.__min = SON(spec)
+        return self
+
     def sort(self, key_or_list, direction=None):
         """Sorts this cursor's results.
 
-        Takes either a single key and a direction, or a list of (key,
-        direction) pairs. The key(s) must be an instance of ``(str,
-        unicode)``, and the direction(s) must be one of
-        (:data:`~pymongo.ASCENDING`,
-        :data:`~pymongo.DESCENDING`). Raises
-        :class:`~pymongo.errors.InvalidOperation` if this cursor has
+        Pass a field name and a direction, either
+        :data:`~pymongo.ASCENDING` or :data:`~pymongo.DESCENDING`::
+
+            for doc in collection.find().sort('field', pymongo.ASCENDING):
+                print(doc)
+
+        To sort by multiple fields, pass a list of (key, direction) pairs::
+
+            for doc in collection.find().sort([
+                    ('field1', pymongo.ASCENDING),
+                    ('field2', pymongo.DESCENDING)]):
+                print(doc)
+
+        Beginning with MongoDB version 2.6, text search results can be
+        sorted by relevance::
+
+            cursor = db.test.find(
+                {'$text': {'$search': 'some words'}},
+                {'score': {'$meta': 'textScore'}})
+
+            # Sort by 'score' field.
+            cursor.sort([('score', {'$meta': 'textScore'})])
+
+            for doc in cursor:
+                print(doc)
+
+        Raises :class:`~pymongo.errors.InvalidOperation` if this cursor has
         already been used. Only the last :meth:`sort` applied to this
         cursor has any effect.
 
@@ -590,6 +700,12 @@ class Cursor(object):
         `with_limit_and_skip` to ``True`` if that is the desired behavior.
         Raises :class:`~pymongo.errors.OperationFailure` on a database error.
 
+        When used with MongoDB >= 2.6, :meth:`~count` uses any :meth:`~hint`
+        applied to the query. In the following example the hint is passed to
+        the count command:
+
+          collection.find({'field': 'value'}).hint('field_1').count()
+
         With :class:`~pymongo.mongo_replica_set_client.MongoReplicaSetClient`
         or :class:`~pymongo.master_slave_connection.MasterSlaveConnection`,
         if `read_preference` is not
@@ -611,12 +727,16 @@ class Cursor(object):
 
             collection.find({}, network_timeout=1).count()
 
+        .. versionchanged:: 2.8
+           The :meth:`~count` method now supports :meth:`~hint`.
+
         .. versionadded:: 1.1.1
            The `with_limit_and_skip` parameter.
            :meth:`~pymongo.cursor.Cursor.__len__` was deprecated in favor of
            calling :meth:`count` with `with_limit_and_skip` set to ``True``.
         """
-        self.__check_not_command_cursor('count')
+        if not isinstance(with_limit_and_skip, bool):
+            raise TypeError("with_limit_and_skip must be an instance of bool")
         command = {"query": self.__spec, "fields": self.__fields}
 
         command['read_preference'] = self.__read_preference
@@ -626,6 +746,13 @@ class Cursor(object):
         command['slave_okay'] = self.__slave_okay
         use_master = not self.__slave_okay and not self.__read_preference
         command['_use_master'] = use_master
+        if self.__max_time_ms is not None:
+            command["maxTimeMS"] = self.__max_time_ms
+        if self.__comment:
+            command['$comment'] = self.__comment
+
+        if self.__hint is not None:
+            command['hint'] = self.__hint
 
         if with_limit_and_skip:
             if self.__limit:
@@ -637,6 +764,7 @@ class Cursor(object):
         r = database.command("count", self.__collection.name,
                              allowable_errors=["ns missing"],
                              uuid_subtype=self.__uuid_subtype,
+                             compile_re=self.__compile_re,
                              **command)
         if r.get("errmsg", "") == "ns missing":
             return 0
@@ -665,7 +793,6 @@ class Cursor(object):
 
         .. versionadded:: 1.2
         """
-        self.__check_not_command_cursor('distinct')
         if not isinstance(key, basestring):
             raise TypeError("key must be an instance "
                             "of %s" % (basestring.__name__,))
@@ -681,11 +808,16 @@ class Cursor(object):
         options['slave_okay'] = self.__slave_okay
         use_master = not self.__slave_okay and not self.__read_preference
         options['_use_master'] = use_master
+        if self.__max_time_ms is not None:
+            options['maxTimeMS'] = self.__max_time_ms
+        if self.__comment:
+            options['$comment'] = self.__comment
 
         database = self.__collection.database
         return database.command("distinct",
                                 self.__collection.name,
                                 uuid_subtype=self.__uuid_subtype,
+                                compile_re=self.__compile_re,
                                 **options)["values"]
 
     def explain(self):
@@ -693,7 +825,6 @@ class Cursor(object):
 
         .. mongodoc:: explain
         """
-        self.__check_not_command_cursor('explain')
         c = self.clone()
         c.__explain = True
 
@@ -715,20 +846,40 @@ class Cursor(object):
 
         `index` should be an index as passed to
         :meth:`~pymongo.collection.Collection.create_index`
-        (e.g. ``[('field', ASCENDING)]``). If `index`
-        is ``None`` any existing hints for this query are cleared. The
-        last hint applied to this cursor takes precedence over all
-        others.
+        (e.g. ``[('field', ASCENDING)]``) or the name of the index.
+        If `index` is ``None`` any existing hint for this query is
+        cleared. The last hint applied to this cursor takes precedence
+        over all others.
 
         :Parameters:
           - `index`: index to hint on (as an index specifier)
+
+        .. versionchanged:: 2.8
+           The :meth:`~hint` method accepts the name of the index.
         """
         self.__check_okay_to_chain()
         if index is None:
             self.__hint = None
             return self
 
-        self.__hint = helpers._index_document(index)
+        if isinstance(index, basestring):
+            self.__hint = index
+        else:
+            self.__hint = helpers._index_document(index)
+        return self
+
+    def comment(self, comment):
+        """Adds a 'comment' to the cursor.
+
+        http://docs.mongodb.org/manual/reference/operator/comment/
+
+        :Parameters:
+          - `comment`: A string or document
+
+        .. versionadded:: 2.7
+        """
+        self.__check_okay_to_chain()
+        self.__comment = comment
         return self
 
     def where(self, code):
@@ -790,20 +941,41 @@ class Cursor(object):
                 # due to a socket timeout.
                 self.__killed = True
                 raise
-        else: # exhaust cursor - no getMore message
-            response = client._exhaust_next(self.__exhaust_mgr.sock)
+        else:
+            # Exhaust cursor - no getMore message.
+            try:
+                response = client._exhaust_next(self.__exhaust_mgr.sock)
+            except AutoReconnect:
+                self.__killed = True
+                self.__exhaust_mgr.error()
+                raise
 
         try:
             response = helpers._unpack_response(response, self.__id,
                                                 self.__as_class,
                                                 self.__tz_aware,
-                                                self.__uuid_subtype)
+                                                self.__uuid_subtype,
+                                                self.__compile_re)
+        except OperationFailure:
+            self.__killed = True
+            # Make sure exhaust socket is returned immediately, if necessary.
+            self.__die()
+            # If this is a tailable cursor the error is likely
+            # due to capped collection roll over. Setting
+            # self.__killed to True ensures Cursor.alive will be
+            # False. No need to re-raise.
+            if self.__query_flags & _QUERY_OPTIONS["tailable_cursor"]:
+                return
+            raise
         except AutoReconnect:
             # Don't send kill cursors to another server after a "not master"
             # error. It's completely pointless.
             self.__killed = True
+            # Make sure exhaust socket is returned immediately, if necessary.
+            self.__die()
             client.disconnect()
             raise
+
         self.__id = response["cursor_id"]
 
         # starting from doesn't get set on getmore's for tailable cursors
@@ -921,16 +1093,16 @@ class Cursor(object):
 
         .. versionadded:: 2.4
         """
-        return self.__clone(deepcopy=False)
+        return self._clone(deepcopy=False)
 
     def __deepcopy__(self, memo):
         """Support function for `copy.deepcopy()`.
 
         .. versionadded:: 2.4
         """
-        return self.__clone(deepcopy=True)
+        return self._clone(deepcopy=True)
 
-    def __deepcopy(self, x, memo=None):
+    def _deepcopy(self, x, memo=None):
         """Deepcopy helper for the data dictionary or list.
 
         Regular expressions cannot be deep copied but as they are immutable we
@@ -950,7 +1122,7 @@ class Cursor(object):
 
         for key, value in iterator:
             if isinstance(value, (dict, list)) and not isinstance(value, SON):
-                value = self.__deepcopy(value, memo)
+                value = self._deepcopy(value, memo)
             elif not isinstance(value, RE_TYPE):
                 value = copy.deepcopy(value, memo)
 
