@@ -23,12 +23,14 @@ sys.path[0:0] = [""]
 
 from bson import InvalidDocument, SON
 from pymongo.errors import BulkWriteError, InvalidOperation, OperationFailure
-from test import version
+from pymongo.mongo_client import _partition_node, MongoClient
+from test import version, skip_restricted_localhost
 from test.test_client import get_client
-from test.utils import (oid_generated_on_client,
-                        remove_all_users,
-                        server_started_with_auth,
-                        server_started_with_nojournal)
+from test.utils import oid_generated_on_client, get_command_line
+
+
+setUpModule = skip_restricted_localhost
+
 
 class BulkTestBase(unittest.TestCase):
 
@@ -600,6 +602,28 @@ class TestBulk(BulkTestBase):
 
         self.assertEqual(1, self.coll.find({'x': 1}).count())
 
+    def test_client_generated_upsert_id(self):
+        batch = self.coll.initialize_ordered_bulk_op()
+        batch.find({'_id': 0}).upsert().update_one({'$set': {'a': 0}})
+        batch.find({'a': 1}).upsert().replace_one({'_id': 1})
+        if not version.at_least(self.coll.database.connection, (2, 6, 0)):
+            # This case is only possible in MongoDB versions before 2.6.
+            batch.find({'_id': 3}).upsert().replace_one({'_id': 2})
+        else:
+            # This is just here to make the counts right in all cases.
+            batch.find({'_id': 2}).upsert().replace_one({'_id': 2})
+        result = batch.execute()
+        self.assertEqualResponse(
+            {'nMatched': 0,
+             'nModified': 0,
+             'nUpserted': 3,
+             'nInserted': 0,
+             'nRemoved': 0,
+             'upserted': [{'index': 0, '_id': 0},
+                          {'index': 1, '_id': 1},
+                          {'index': 2, '_id': 2}]},
+            result)
+
     def test_single_ordered_batch(self):
         batch = self.coll.initialize_ordered_bulk_op()
         batch.insert({'a': 1})
@@ -893,10 +917,47 @@ class TestBulkWriteConcern(BulkTestBase):
         ismaster = client.test.command('ismaster')
         self.is_repl = bool(ismaster.get('setName'))
         self.w = len(ismaster.get("hosts", []))
+
+        self.secondary = None
+        if self.w > 1:
+            for member in ismaster['hosts']:
+                if member != ismaster['primary']:
+                    host, port = _partition_node(member)
+                    self.secondary = MongoClient(host, port)
+                    break
+
         self.client = client
         self.coll = client.pymongo_test.test
         self.coll.remove()
 
+        # We tested wtimeout errors by specifying a write concern greater than
+        # the number of members, but in MongoDB 2.7.8+ this causes a different
+        # sort of error, "Not enough data-bearing nodes". In recent servers we
+        # use a failpoint to pause replication on a secondary.
+        self.need_replication_stopped = version.at_least(self.client,
+                                                         (2, 7, 8))
+
+        self.test_commands_enabled = ("enableTestCommands=1"
+                                      in get_command_line(self.client)["argv"])
+
+    def cause_wtimeout(self, batch):
+        if self.need_replication_stopped:
+            if not self.test_commands_enabled:
+                raise SkipTest("Test commands must be enabled.")
+
+            self.secondary.admin.command('configureFailPoint',
+                                         'rsSyncApplyStop',
+                                         mode='alwaysOn')
+
+            try:
+                return batch.execute({'w': self.w, 'wtimeout': 1})
+            finally:
+                self.secondary.admin.command('configureFailPoint',
+                                             'rsSyncApplyStop',
+                                             mode='off')
+        else:
+            return batch.execute({'w': self.w + 1, 'wtimeout': 1})
+
     def test_fsync_and_j(self):
         if not version.at_least(self.client, (1, 8, 2)):
             raise SkipTest("Need at least MongoDB 1.8.2")
@@ -922,7 +983,7 @@ class TestBulkWriteConcern(BulkTestBase):
         # Replication wtimeout is a 'soft' error.
         # It shouldn't stop batch processing.
         try:
-            batch.execute({'w': self.w + 1, 'wtimeout': 1})
+            self.cause_wtimeout(batch)
         except BulkWriteError, exc:
             result = exc.details
             self.assertEqual(exc.code, 65)
@@ -959,7 +1020,7 @@ class TestBulkWriteConcern(BulkTestBase):
             batch.insert({'a': 1})
             batch.insert({'a': 2})
             try:
-                batch.execute({'w': self.w + 1, 'wtimeout': 1})
+                self.cause_wtimeout(batch)
             except BulkWriteError, exc:
                 result = exc.details
                 self.assertEqual(exc.code, 65)
@@ -1003,7 +1064,7 @@ class TestBulkWriteConcern(BulkTestBase):
         # Replication wtimeout is a 'soft' error.
         # It shouldn't stop batch processing.
         try:
-            batch.execute({'w': self.w + 1, 'wtimeout': 1})
+            self.cause_wtimeout(batch)
         except BulkWriteError, exc:
             result = exc.details
             self.assertEqual(exc.code, 65)
@@ -1030,7 +1091,7 @@ class TestBulkWriteConcern(BulkTestBase):
             batch.insert({'a': 1})
             batch.insert({'a': 2})
             try:
-                batch.execute({'w': self.w + 1, 'wtimeout': 1})
+                self.cause_wtimeout(batch)
             except BulkWriteError, exc:
                 result = exc.details
                 self.assertEqual(exc.code, 65)
@@ -1113,63 +1174,5 @@ class TestBulkNoResults(BulkTestBase):
         self.assertTrue(self.coll.find_one({'_id': 1}) is None)
 
 
-class TestBulkAuthorization(BulkTestBase):
-
-    def setUp(self):
-        super(TestBulkAuthorization, self).setUp()
-        self.client = client = get_client()
-        if (not server_started_with_auth(client)
-                or not version.at_least(client, (2, 5, 3))):
-            raise SkipTest('Need at least MongoDB 2.5.3 with auth')
-
-        db = client.pymongo_test
-        self.coll = db.test
-        self.coll.remove()
-
-        db.add_user('dbOwner', 'pw', roles=['dbOwner'])
-        db.authenticate('dbOwner', 'pw')
-        db.add_user('readonly', 'pw', roles=['read'])
-        db.command(
-            'createRole', 'noremove',
-            privileges=[{
-                'actions': ['insert', 'update', 'find'],
-                'resource': {'db': 'pymongo_test', 'collection': 'test'}
-            }],
-            roles=[])
-
-        db.add_user('noremove', 'pw', roles=['noremove'])
-        db.logout()
-
-    def test_readonly(self):
-        # We test that an authorization failure aborts the batch and is raised
-        # as OperationFailure.
-        db = self.client.pymongo_test
-        db.authenticate('readonly', 'pw')
-        bulk = self.coll.initialize_ordered_bulk_op()
-        bulk.insert({'x': 1})
-        self.assertRaises(OperationFailure, bulk.execute)
-
-    def test_no_remove(self):
-        # We test that an authorization failure aborts the batch and is raised
-        # as OperationFailure.
-        db = self.client.pymongo_test
-        db.authenticate('noremove', 'pw')
-        bulk = self.coll.initialize_ordered_bulk_op()
-        bulk.insert({'x': 1})
-        bulk.find({'x': 2}).upsert().replace_one({'x': 2})
-        bulk.find({}).remove()  # Prohibited.
-        bulk.insert({'x': 3})   # Never attempted.
-        self.assertRaises(OperationFailure, bulk.execute)
-        self.assertEqual(set([1, 2]), set(self.coll.distinct('x')))
-
-    def tearDown(self):
-        db = self.client.pymongo_test
-        db.logout()
-        db.authenticate('dbOwner', 'pw')
-        db.command('dropRole', 'noremove')
-        remove_all_users(db)
-        db.logout()
-
-
 if __name__ == "__main__":
     unittest.main()
