@@ -37,12 +37,15 @@ from pymongo.database import Database
 from pymongo.errors import (InvalidOperation,
                             OperationFailure,
                             ExecutionTimeout)
-from test import version
+from test import version, skip_restricted_localhost
 from test.test_client import get_client
 from test.utils import (catch_warnings, is_mongos,
                         get_command_line, server_started_with_auth)
 
 
+setUpModule = skip_restricted_localhost
+
+
 class TestCursor(unittest.TestCase):
 
     def setUp(self):
@@ -131,16 +134,12 @@ class TestCursor(unittest.TestCase):
 
     def test_explain(self):
         a = self.db.test.find()
-        b = a.explain()
+        a.explain()
         for _ in a:
             break
-        c = a.explain()
-        del b["millis"]
-        b.pop("oldPlan", None)
-        del c["millis"]
-        c.pop("oldPlan", None)
-        self.assertEqual(b, c)
-        self.assertTrue("cursor" in b)
+        b = a.explain()
+        # "cursor" pre MongoDB 2.7.6, "executionStats" post
+        self.assertTrue("cursor" in b or "executionStats" in b)
 
     def test_hint(self):
         db = self.db
@@ -157,15 +156,13 @@ class TestCursor(unittest.TestCase):
                           db.test.find({"num": 17, "foo": 17})
                           .hint([("foo", ASCENDING)]).explain)
 
-        index = db.test.create_index("num")
+        spec = [("num", DESCENDING)]
+        index = db.test.create_index(spec)
 
-        spec = [("num", ASCENDING)]
-        self.assertEqual(db.test.find({}).explain()["cursor"], "BasicCursor")
-        self.assertEqual(db.test.find({}).hint(spec).explain()["cursor"],
-                         "BtreeCursor %s" % index)
-        self.assertEqual(db.test.find({}).hint(spec).hint(None)
-                         .explain()["cursor"],
-                         "BasicCursor")
+        first = db.test.find().next()
+        self.assertEqual(0, first.get('num'))
+        first = db.test.find().hint(spec).next()
+        self.assertEqual(99, first.get('num'))
         self.assertRaises(OperationFailure,
                           db.test.find({"num": 17, "foo": 17})
                           .hint([("foo", ASCENDING)]).explain)
@@ -176,7 +173,18 @@ class TestCursor(unittest.TestCase):
             break
         self.assertRaises(InvalidOperation, a.hint, spec)
 
-        self.assertRaises(TypeError, db.test.find().hint, index)
+    def test_hint_by_name(self):
+        db = self.db
+        db.test.drop()
+
+        for i in range(100):
+            db.test.insert({'i': i})
+
+        db.test.create_index([('i', DESCENDING)], name='fooindex')
+        first = db.test.find().next()
+        self.assertEqual(0, first.get('i'))
+        first = db.test.find().hint('fooindex').next()
+        self.assertEqual(99, first.get('i'))
 
     def test_limit(self):
         db = self.db
@@ -498,6 +506,40 @@ class TestCursor(unittest.TestCase):
 
         self.assertEqual(0, db.test.acollectionthatdoesntexist.find().count())
 
+    def test_count_with_hint(self):
+        collection = self.db.test
+        collection.drop()
+
+        collection.save({'i': 1})
+        collection.save({'i': 2})
+        self.assertEqual(2, collection.find().count())
+
+        collection.create_index([('i', 1)])
+
+        self.assertEqual(1, collection.find({'i': 1}).hint("_id_").count())
+        self.assertEqual(2, collection.find().hint("_id_").count())
+
+        if version.at_least(self.client, (2, 6, 0)):
+            # Count supports hint
+            self.assertRaises(OperationFailure,
+                              collection.find({'i': 1}).hint("BAD HINT").count)
+        else:
+            # Hint is ignored
+            self.assertEqual(
+                1, collection.find({'i': 1}).hint("BAD HINT").count())
+
+        # Create a sparse index which should have no entries.
+        collection.create_index([('x', 1)], sparse=True)
+
+        if version.at_least(self.client, (2, 6, 0)):
+            # Count supports hint
+            self.assertEqual(0, collection.find({'i': 1}).hint("x_1").count())
+        else:
+            # Hint is ignored
+            self.assertEqual(1, collection.find({'i': 1}).hint("x_1").count())
+
+        self.assertEqual(2, collection.find().hint("x_1").count())
+
     def test_where(self):
         db = self.db
         db.test.drop()
