@@ -32,11 +32,15 @@ from gridfs.errors import (FileExists, NoFile)
 from pymongo.errors import ConnectionFailure
 from pymongo.mongo_client import MongoClient
 from pymongo.read_preferences import ReadPreference
+from test import skip_restricted_localhost
 from test.test_client import get_client
 from test.test_replica_set_client import TestReplicaSetClientBase
 from test.utils import catch_warnings, joinall
 
 
+setUpModule = skip_restricted_localhost
+
+
 class JustWrite(threading.Thread):
 
     def __init__(self, fs, n):
@@ -396,11 +400,35 @@ class TestGridfs(unittest.TestCase):
         cursor.close()
         self.assertRaises(TypeError, self.fs.find, {}, {"_id": True})
 
+    def test_gridfs_find_one(self):
+        self.assertEqual(None, self.fs.find_one())
+
+        id1 = self.fs.put(b('test1'), filename='file1')
+        self.assertEqual(b('test1'), self.fs.find_one().read())
+
+        id2 = self.fs.put(b('test2'), filename='file2', meta='data')
+        self.assertEqual(b('test1'), self.fs.find_one(id1).read())
+        self.assertEqual(b('test2'), self.fs.find_one(id2).read())
+
+        self.assertEqual(b('test1'),
+                         self.fs.find_one({'filename': 'file1'}).read())
+
+        self.assertEqual('data', self.fs.find_one(id2).meta)
+
+    def test_grid_in_non_int_chunksize(self):
+        # Lua, and perhaps other buggy GridFS clients, store size as a float.
+        data = b('data')
+        self.fs.put(data, filename='f')
+        self.db.fs.files.update({'filename': 'f'},
+                                {'$set': {'chunkSize': 100.0}})
+
+        self.assertEqual(data, self.fs.get_version('f').read())
+
 
 class TestGridfsReplicaSet(TestReplicaSetClientBase):
     def test_gridfs_replica_set(self):
         rsc = self._get_client(
-            w=self.w, wtimeout=5000,
+            w=self.w, wtimeout=30000,
             read_preference=ReadPreference.SECONDARY)
 
         try:
