@@ -23,13 +23,16 @@ sys.path[0:0] = [""]
 
 from nose.plugins.skip import SkipTest
 
-from test import host, port
+from test import host, port, skip_restricted_localhost
 from test.test_pooling_base import (
     _TestPooling, _TestMaxPoolSize, _TestMaxOpenSockets,
-    _TestPoolSocketSharing, _TestWaitQueueMultiple, one)
+    _TestWaitQueueMultiple, one)
 from test.utils import get_pool
 
 
+setUpModule = skip_restricted_localhost
+
+
 class TestPoolingThreads(_TestPooling, unittest.TestCase):
     use_greenlets = False
 
@@ -165,10 +168,6 @@ class TestMaxPoolSizeThreads(_TestMaxPoolSize, unittest.TestCase):
     use_greenlets = False
 
 
-class TestPoolSocketSharingThreads(_TestPoolSocketSharing, unittest.TestCase):
-    use_greenlets = False
-
-
 class TestMaxOpenSocketsThreads(_TestMaxOpenSockets, unittest.TestCase):
     use_greenlets = False
 
