@@ -134,7 +134,8 @@ class NonUnique(MongoThread):
         for _ in xrange(N):
             self.client.start_request()
             self.db.unique.insert({"_id": "jesse"}, w=0)
-            self.ut.assertNotEqual(None, self.db.error())
+            self.ut.assertNotEqual(None,
+                                   self.db.command('getlasterror').get('err'))
             self.client.end_request()
 
 
@@ -152,7 +153,7 @@ class NoRequest(MongoThread):
         errors = 0
         for _ in xrange(N):
             self.db.unique.insert({"_id": "jesse"}, w=0)
-            if not self.db.error():
+            if not self.db.command("getlasterror").get("err"):
                 errors += 1
 
         self.client.end_request()
@@ -1141,99 +1142,3 @@ class _TestWaitQueueMultiple(_TestPoolingBase):
 
         for socket_info in socks:
             socket_info.close()
-
-
-class _TestPoolSocketSharing(_TestPoolingBase):
-    """Directly test that two simultaneous operations don't share a socket. To
-    be run both with threads and with greenlets.
-    """
-    def _test_pool(self, use_request):
-        """
-        Test that the connection pool prevents both threads and greenlets from
-        using a socket at the same time.
-
-        Sequence:
-        gr0: start a slow find()
-        gr1: start a fast find()
-        gr1: get results
-        gr0: get results
-        """
-        cx = get_client(
-            use_greenlets=self.use_greenlets,
-            auto_start_request=False
-        )
-
-        db = cx.pymongo_test
-        db.test.remove()
-        db.test.insert({'_id': 1})
-
-        history = []
-
-        def find_fast():
-            if use_request:
-                cx.start_request()
-
-            history.append('find_fast start')
-
-            # With greenlets and the old connection._Pool, this would throw
-            # AssertionError: "This event is already used by another
-            # greenlet"
-            self.assertEqual({'_id': 1}, db.test.find_one())
-            history.append('find_fast done')
-
-            if use_request:
-                cx.end_request()
-
-        def find_slow():
-            if use_request:
-                cx.start_request()
-
-            history.append('find_slow start')
-
-            # Javascript function that pauses N seconds per document
-            fn = delay(10)
-            if (is_mongos(db.connection) or not
-                version.at_least(db.connection, (1, 7, 2))):
-                # mongos doesn't support eval so we have to use $where
-                # which is less reliable in this context.
-                self.assertEqual(1, db.test.find({"$where": fn}).count())
-            else:
-                # 'nolock' allows find_fast to start and finish while we're
-                # waiting for this to complete.
-                self.assertEqual({'ok': 1.0, 'retval': True},
-                                 db.command('eval', fn, nolock=True))
-
-            history.append('find_slow done')
-
-            if use_request:
-                cx.end_request()
-
-        if self.use_greenlets:
-            gr0, gr1 = Greenlet(find_slow), Greenlet(find_fast)
-            gr0.start()
-            gr1.start_later(.1)
-        else:
-            gr0 = threading.Thread(target=find_slow)
-            gr0.setDaemon(True)
-            gr1 = threading.Thread(target=find_fast)
-            gr1.setDaemon(True)
-
-            gr0.start()
-            time.sleep(.1)
-            gr1.start()
-
-        gr0.join()
-        gr1.join()
-
-        self.assertEqual([
-            'find_slow start',
-            'find_fast start',
-            'find_fast done',
-            'find_slow done',
-        ], history)
-
-    def test_pool(self):
-        self._test_pool(use_request=False)
-
-    def test_pool_request(self):
-        self._test_pool(use_request=True)
