@@ -26,6 +26,7 @@ import thread
 import threading
 import traceback
 import unittest
+import warnings
 
 sys.path[0:0] = [""]
 
@@ -35,7 +36,7 @@ from bson.son import SON
 from bson.tz_util import utc
 from pymongo.mongo_client import MongoClient
 from pymongo.read_preferences import ReadPreference
-from pymongo.member import PRIMARY, SECONDARY, OTHER
+from pymongo.member import SECONDARY
 from pymongo.mongo_replica_set_client import MongoReplicaSetClient
 from pymongo.mongo_replica_set_client import _partition_node, have_gevent
 from pymongo.database import Database
@@ -45,13 +46,15 @@ from pymongo.errors import (AutoReconnect,
                             ConnectionFailure,
                             InvalidName,
                             OperationFailure, InvalidOperation)
-from test import version, port, pair
+from test import version, port, pair, skip_restricted_localhost, auth_context
 from test.pymongo_mocks import MockReplicaSetClient
 from test.utils import (
     delay, assertReadFrom, assertReadFromAll, read_from_which_host,
-    remove_all_users, assertRaisesExactly, TestRequestMixin, one,
-    server_started_with_auth, pools_from_rs_client, get_pool,
-    _TestLazyConnectMixin)
+    assertRaisesExactly, TestRequestMixin, one, pools_from_rs_client, get_pool,
+    _TestLazyConnectMixin, _TestExhaustCursorMixin, catch_warnings)
+
+
+setUpModule = skip_restricted_localhost
 
 
 class TestReplicaSetClientAgainstStandalone(unittest.TestCase):
@@ -82,7 +85,10 @@ class TestReplicaSetClientBase(unittest.TestCase):
             self.arbiters = set([_partition_node(h)
                                  for h in response.get("arbiters", [])])
 
-            repl_set_status = client.admin.command('replSetGetStatus')
+            # Cannot run replSetGetStatus in MongoDB >= 2.7.1 under auth once a
+            # user has been added.
+            repl_set_status = auth_context.client.admin.command(
+                'replSetGetStatus')
             primary_info = [
                 m for m in repl_set_status['members']
                 if m['stateStr'] == 'PRIMARY'
@@ -121,6 +127,43 @@ class TestReplicaSetClient(TestReplicaSetClientBase, TestRequestMixin):
             standardMsg = '%r is not an instance of %r' % (obj, cls)
             self.fail(self._formatMessage(msg, standardMsg))
 
+    def test_keyword_arg_defaults(self):
+        client = MongoReplicaSetClient(socketTimeoutMS=None,
+                                       connectTimeoutMS=20000,
+                                       waitQueueTimeoutMS=None,
+                                       waitQueueMultiple=None,
+                                       socketKeepAlive=False,
+                                       auto_start_request=False,
+                                       use_greenlets=False,
+                                       replicaSet='myreplset',  # Required
+                                       read_preference=ReadPreference.PRIMARY,
+                                       tag_sets=[{}],
+                                       ssl=False,
+                                       ssl_keyfile=None,
+                                       ssl_certfile=None,
+                                       ssl_ca_certs=None,
+                                       _connect=False)
+        self.assertEqual(None,
+                         client._MongoReplicaSetClient__net_timeout)
+        # socket.Socket.settimeout takes a float in seconds
+        self.assertEqual(20.0,
+                         client._MongoReplicaSetClient__conn_timeout)
+        self.assertEqual(None,
+                         client._MongoReplicaSetClient__wait_queue_timeout)
+        self.assertEqual(None,
+                         client._MongoReplicaSetClient__wait_queue_multiple)
+        self.assertFalse(client._MongoReplicaSetClient__socket_keepalive)
+        self.assertFalse(client.auto_start_request)
+        self.assertFalse(client.use_greenlets)
+        self.assertEqual('myreplset',
+                         client._MongoReplicaSetClient__name)
+        self.assertEqual(ReadPreference.PRIMARY, client.read_preference)
+        self.assertEqual([{}], client.tag_sets)
+        self.assertFalse(client._MongoReplicaSetClient__use_ssl)
+        self.assertEqual(None, client._MongoReplicaSetClient__ssl_keyfile)
+        self.assertEqual(None, client._MongoReplicaSetClient__ssl_certfile)
+        self.assertEqual(None, client._MongoReplicaSetClient__ssl_ca_certs)
+
     def test_init_disconnected(self):
         c = self._get_client(_connect=False)
 
@@ -167,44 +210,6 @@ class TestReplicaSetClient(TestReplicaSetClientBase, TestRequestMixin):
 
         self.assertRaises(ConnectionFailure, c.pymongo_test.test.find_one)
 
-    def test_init_disconnected_with_auth_failure(self):
-        c = MongoReplicaSetClient(
-            "mongodb://user:pass@somedomainthatdoesntexist", replicaSet="rs",
-            connectTimeoutMS=1, _connect=False)
-
-        self.assertRaises(ConnectionFailure, c.pymongo_test.test.find_one)
-
-    def test_init_disconnected_with_auth(self):
-        c = self._get_client()
-        if not server_started_with_auth(c):
-            raise SkipTest('Authentication is not enabled on server')
-
-        c.admin.add_user("admin", "pass")
-        c.admin.authenticate("admin", "pass")
-        try:
-            c.pymongo_test.add_user("user", "pass", roles=['readWrite', 'userAdmin'])
-
-            # Auth with lazy connection.
-            host = one(self.hosts)
-            uri = "mongodb://user:pass@%s:%d/pymongo_test?replicaSet=%s" % (
-                host[0], host[1], self.name)
-
-            authenticated_client = MongoReplicaSetClient(uri, _connect=False)
-            authenticated_client.pymongo_test.test.find_one()
-
-            # Wrong password.
-            bad_uri = "mongodb://user:wrong@%s:%d/pymongo_test?replicaSet=%s" % (
-                host[0], host[1], self.name)
-
-            bad_client = MongoReplicaSetClient(bad_uri, _connect=False)
-            self.assertRaises(
-                OperationFailure, bad_client.pymongo_test.test.find_one)
-
-        finally:
-            # Clean up.
-            remove_all_users(c.pymongo_test)
-            remove_all_users(c.admin)
-
     def test_connect(self):
         assertRaisesExactly(ConnectionFailure, MongoReplicaSetClient,
                           "somedomainthatdoesntexist.org:27017",
@@ -340,20 +345,6 @@ class TestReplicaSetClient(TestReplicaSetClientBase, TestRequestMixin):
         finally:
             socket.socket.sendall = old_sendall
 
-    def test_lazy_auth_raises_operation_failure(self):
-        # Check if we have the prerequisites to run this test.
-        c = self._get_client()
-        if not server_started_with_auth(c):
-            raise SkipTest('Authentication is not enabled on server')
-
-        lazy_client = MongoReplicaSetClient(
-            "mongodb://user:wrong@%s/pymongo_test" % pair,
-            replicaSet=self.name,
-            _connect=False)
-
-        assertRaisesExactly(
-            OperationFailure, lazy_client.test.collection.find_one)
-
     def test_operations(self):
         c = self._get_client()
 
@@ -427,72 +418,25 @@ class TestReplicaSetClient(TestReplicaSetClientBase, TestRequestMixin):
 
     def test_copy_db(self):
         c = self._get_client()
-        # We test copy twice; once starting in a request and once not. In
-        # either case the copy should succeed (because it starts a request
-        # internally) and should leave us in the same state as before the copy.
-        c.start_request()
+        ctx = catch_warnings()
+        try:
+            warnings.simplefilter("ignore", DeprecationWarning)
+            self.assertRaises(TypeError, c.copy_database, 4, "foo")
+            self.assertRaises(TypeError, c.copy_database, "foo", 4)
+            self.assertRaises(InvalidName, c.copy_database, "foo", "$foo")
 
-        self.assertRaises(TypeError, c.copy_database, 4, "foo")
-        self.assertRaises(TypeError, c.copy_database, "foo", 4)
+            c.pymongo_test.test.drop()
+            c.pymongo_test.test.insert({"foo": "bar"})
 
-        self.assertRaises(InvalidName, c.copy_database, "foo", "$foo")
-
-        c.pymongo_test.test.drop()
-        c.drop_database("pymongo_test1")
-        c.drop_database("pymongo_test2")
-
-        c.pymongo_test.test.insert({"foo": "bar"})
-
-        self.assertFalse("pymongo_test1" in c.database_names())
-        self.assertFalse("pymongo_test2" in c.database_names())
-
-        c.copy_database("pymongo_test", "pymongo_test1")
-        # copy_database() didn't accidentally end the request
-        self.assertTrue(c.in_request())
-
-        self.assertTrue("pymongo_test1" in c.database_names())
-        self.assertEqual("bar", c.pymongo_test1.test.find_one()["foo"])
-
-        c.end_request()
-
-        self.assertFalse(c.in_request())
-        c.copy_database("pymongo_test", "pymongo_test2", pair)
-        # copy_database() didn't accidentally restart the request
-        self.assertFalse(c.in_request())
-
-        time.sleep(1)
-
-        self.assertTrue("pymongo_test2" in c.database_names())
-        self.assertEqual("bar", c.pymongo_test2.test.find_one()["foo"])
-
-        if version.at_least(c, (1, 3, 3, 1)) and server_started_with_auth(c):
             c.drop_database("pymongo_test1")
+            self.assertFalse("pymongo_test1" in c.database_names())
 
-            c.admin.add_user("admin", "password")
-            c.admin.authenticate("admin", "password")
-            try:
-                c.pymongo_test.add_user("mike", "password")
-
-                self.assertRaises(OperationFailure, c.copy_database,
-                                  "pymongo_test", "pymongo_test1",
-                                  username="foo", password="bar")
-                self.assertFalse("pymongo_test1" in c.database_names())
-
-                self.assertRaises(OperationFailure, c.copy_database,
-                                  "pymongo_test", "pymongo_test1",
-                                  username="mike", password="bar")
-                self.assertFalse("pymongo_test1" in c.database_names())
-
-                c.copy_database("pymongo_test", "pymongo_test1",
-                                username="mike", password="password")
-                self.assertTrue("pymongo_test1" in c.database_names())
-                res = c.pymongo_test1.test.find_one(_must_use_master=True)
-                self.assertEqual("bar", res["foo"])
-            finally:
-                # Cleanup
-                remove_all_users(c.pymongo_test)
-                c.admin.remove_user("admin")
-        c.close()
+            c.copy_database("pymongo_test", "pymongo_test1")
+            self.assertTrue("pymongo_test1" in c.database_names())
+            self.assertEqual("bar", c.pymongo_test1.test.find_one()["foo"])
+            c.drop_database("pymongo_test1")
+        finally:
+            ctx.exit()
 
     def test_get_default_database(self):
         host = one(self.hosts)
@@ -747,6 +691,11 @@ class TestReplicaSetClient(TestReplicaSetClientBase, TestRequestMixin):
         self.assertEqual(pool.wait_queue_multiple, 2)
         self.assertEqual(pool._socket_semaphore.waiter_semaphore.counter, 6)
 
+    def test_socketKeepAlive(self):
+        client = self._get_client(socketKeepAlive=True)
+        pool = get_pool(client)
+        self.assertTrue(pool.socket_keepalive)
+
     def test_tz_aware(self):
         self.assertRaises(ConfigurationError, MongoReplicaSetClient,
                           tz_aware='foo', replicaSet=self.name)
@@ -1237,5 +1186,12 @@ class TestReplicaSetClientMaxWriteBatchSize(unittest.TestCase):
         self.assertEqual(c.max_write_batch_size, 2)
 
 
+class TestReplicaSetClientExhaustCursor(
+        _TestExhaustCursorMixin,
+        TestReplicaSetClientBase):
+
+    # Base class implements _get_client already.
+    pass
+
 if __name__ == "__main__":
     unittest.main()
