@@ -21,9 +21,13 @@ sys.path[0:0] = [""]
 
 from pymongo.errors import ConfigurationError, ConnectionFailure
 from pymongo import ReadPreference
+from test import skip_restricted_localhost
 from test.pymongo_mocks import MockClient, MockReplicaSetClient
 
 
+setUpModule = skip_restricted_localhost
+
+
 class TestSecondaryBecomesStandalone(unittest.TestCase):
     # An administrator removes a secondary from a 3-node set and
     # brings it back up as standalone, without updating the other
