@@ -67,6 +67,15 @@ class TestSON(unittest.TestCase):
                                      ('mike', 'awesome'),
                                      ('hello', 'world'))))
 
+        # Embedded SON.
+        d4 = SON([('blah', {'foo': SON()})])
+        self.assertEqual(d4, {'blah': {'foo': {}}})
+        self.assertEqual(d4, {'blah': {'foo': SON()}})
+        self.assertNotEqual(d4, {'blah': {'foo': []}})
+
+        # Original data unaffected.
+        self.assertEqual(SON, d4['blah']['foo'].__class__)
+
     def test_to_dict(self):
         a1 = SON()
         b2 = SON([("blah", SON())])
@@ -81,6 +90,9 @@ class TestSON(unittest.TestCase):
         self.assertEqual(dict, c3.to_dict()["blah"][0].__class__)
         self.assertEqual(dict, d4.to_dict()["blah"]["foo"].__class__)
 
+        # Original data unaffected.
+        self.assertEqual(SON, d4['blah']['foo'].__class__)
+
     def test_pickle(self):
 
         simple_son = SON([])
@@ -143,6 +155,47 @@ class TestSON(unittest.TestCase):
         self.assertEqual(reflexive_son.keys(), reflexive_son1.keys())
         self.assertEqual(id(reflexive_son1), id(reflexive_son1["reflexive"]))
 
+    def test_iteration(self):
+        """
+        Test __iter__
+        """
+        # test success case
+        test_son = SON([(1, 100), (2, 200), (3, 300)])
+        for ele in test_son:
+            self.assertEqual(ele * 100, test_son[ele])
+
+    def test_contains_has(self):
+        """
+        has_key and __contains__
+        """
+        test_son = SON([(1, 100), (2, 200), (3, 300)])
+        self.assertTrue(1 in test_son)
+        self.assertTrue(2 in test_son, "in failed")
+        self.assertFalse(22 in test_son, "in succeeded when it shouldn't")
+        self.assertTrue(test_son.has_key(2), "has_key failed")
+        self.assertFalse(test_son.has_key(22), "has_key succeeded when it shouldn't")
+
+    def test_clears(self):
+        """
+        Test clear()
+        """
+        test_son = SON([(1, 100), (2, 200), (3, 300)])
+        test_son.clear()
+        self.assertFalse(1 in test_son)
+        self.assertEqual(0, len(test_son))
+        self.assertEqual(0, len(test_son.keys()))
+        self.assertEqual({}, test_son.to_dict())
+
+    def test_len(self):
+        """
+        Test len
+        """
+        test_son = SON()
+        self.assertEqual(0, len(test_son))
+        test_son = SON([(1, 100), (2, 200), (3, 300)])
+        self.assertEqual(3, len(test_son))
+        test_son.popitem()
+        self.assertEqual(2, len(test_son))
 
 if __name__ == "__main__":
     unittest.main()
