@@ -1,4 +1,4 @@
-# Copyright 2009-2012 10gen, Inc.
+# Copyright 2009-2014 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,13 +18,15 @@ import unittest
 import threading
 import traceback
 
-from nose.plugins.skip import SkipTest
-
-from test.utils import server_started_with_auth, joinall, RendezvousThread
+from test import skip_restricted_localhost
+from test.utils import joinall, RendezvousThread
 from test.test_client import get_client
 from test.utils import get_pool
 from pymongo.pool import SocketInfo, _closed
-from pymongo.errors import AutoReconnect, OperationFailure
+from pymongo.errors import AutoReconnect
+
+
+setUpModule = skip_restricted_localhost
 
 
 class AutoAuthenticateThreads(threading.Thread):
@@ -139,8 +141,8 @@ class FindPauseFind(RendezvousThread):
         # acquire a socket
         list(self.collection.find())
 
-        self.pool = get_pool(self.collection.database.connection)
-        socket_info = self.pool._get_request_state()
+        pool = get_pool(self.collection.database.connection)
+        socket_info = pool._get_request_state()
         assert isinstance(socket_info, SocketInfo)
         self.request_sock = socket_info.sock
         assert not _closed(self.request_sock)
@@ -150,12 +152,12 @@ class FindPauseFind(RendezvousThread):
         # because it's not our request socket anymore
         assert _closed(self.request_sock)
 
-        # if disconnect() properly closed all threads' request sockets, then
-        # this won't raise AutoReconnect because it will acquire a new socket
-        assert self.request_sock == self.pool._get_request_state().sock
+        # if disconnect() properly replaced the pool, then this won't raise
+        # AutoReconnect because it will acquire a new socket
         list(self.collection.find())
         assert self.collection.database.connection.in_request()
-        assert self.request_sock != self.pool._get_request_state().sock
+        pool = get_pool(self.collection.database.connection)
+        assert self.request_sock != pool._get_request_state().sock
 
 
 class BaseTestThreads(object):
@@ -258,7 +260,7 @@ class BaseTestThreads(object):
         assert isinstance(socket_info, SocketInfo)
         request_sock = socket_info.sock
 
-        state = FindPauseFind.create_shared_state(nthreads=40)
+        state = FindPauseFind.create_shared_state(nthreads=10)
 
         threads = [
             FindPauseFind(collection, state)
@@ -298,84 +300,9 @@ class BaseTestThreads(object):
             self.assertTrue(t.passed, "%s threw exception" % t)
 
 
-class BaseTestThreadsAuth(object):
-    """
-    Base test class for TestThreadsAuth and TestThreadsAuthReplicaSet. (This is
-    not itself a unittest.TestCase, otherwise it'd be run twice -- once when
-    nose imports this module, and once when nose imports
-    test_threads_replica_set_connection.py, which imports this module.)
-    """
-    def _get_client(self):
-        """
-        Intended for overriding in TestThreadsAuthReplicaSet. This method
-        returns a MongoClient here, and a MongoReplicaSetClient in
-        test_threads_replica_set_connection.py.
-        """
-        # Regular test client
-        return get_client()
-
-    def setUp(self):
-        client = self._get_client()
-        if not server_started_with_auth(client):
-            raise SkipTest("Authentication is not enabled on server")
-        self.client = client
-        self.client.admin.system.users.remove({})
-        self.client.admin.add_user('admin-user', 'password')
-        self.client.admin.authenticate("admin-user", "password")
-        self.client.auth_test.system.users.remove({})
-        self.client.auth_test.add_user("test-user", "password")
-
-    def tearDown(self):
-        # Remove auth users from databases
-        self.client.admin.authenticate("admin-user", "password")
-        self.client.admin.system.users.remove({})
-        self.client.auth_test.system.users.remove({})
-        self.client.drop_database('auth_test')
-        # Clear client reference so that RSC's monitor thread
-        # dies.
-        self.client = None
-
-    def test_auto_auth_login(self):
-        client = self._get_client()
-        self.assertRaises(OperationFailure, client.auth_test.test.find_one)
-
-        # Admin auth
-        client = self._get_client()
-        client.admin.authenticate("admin-user", "password")
-
-        nthreads = 10
-        threads = []
-        for _ in xrange(nthreads):
-            t = AutoAuthenticateThreads(client.auth_test.test, 100)
-            t.start()
-            threads.append(t)
-
-        joinall(threads)
-
-        for t in threads:
-            self.assertTrue(t.success)
-
-        # Database-specific auth
-        client = self._get_client()
-        client.auth_test.authenticate("test-user", "password")
-
-        threads = []
-        for _ in xrange(nthreads):
-            t = AutoAuthenticateThreads(client.auth_test.test, 100)
-            t.start()
-            threads.append(t)
-
-        joinall(threads)
-
-        for t in threads:
-            self.assertTrue(t.success)
-
 class TestThreads(BaseTestThreads, unittest.TestCase):
     pass
 
-class TestThreadsAuth(BaseTestThreadsAuth, unittest.TestCase):
-    pass
-
 
 if __name__ == "__main__":
     unittest.main()
